@@ -1,7 +1,7 @@
 package Switch::Back;
 
 use 5.036;
-our $VERSION = '0.000002';
+our $VERSION = '0.000003';
 
 use experimental qw< builtin refaliasing try >;
 use builtin      qw< true false blessed created_as_number >;
@@ -86,26 +86,27 @@ sub _pure_given_impl { my ($source) = @_;
     state @pure_statements;
     @pure_statements = ();
     state $VALIDATE_PURE_GIVEN = qr{
-        \A  given (?<GIVEN> $OWS  \(
-                (?<JUNC> (?: $OWS (?> any | all | none )  $OWS  => )?+ )
-                             $OWS (?>(?<EXPR> (?&PerlExpression)))
-                             $OWS \)
-                             $OWS (?>(?<BLOCK>  (?&PureBlock)  ))
+        \A  given (?<GIVEN>  (?<ws_post_kw>   $OWS    )  \(
+                             (?<ws_pre_expr>  $OWS    )  (?>(?<EXPR> (?&PerlExpression)))
+                             (?<ws_pre_close> $OWS    )  \)
+                             (?<ws_pre_block> $OWS \{ $OWS )  (?>(?<BLOCK>  (?&PureBlock)  ))  \}
             )
             (?>(?<TRAILING_CODE>  .*  ))
 
             (?(DEFINE)
                 (?<PureBlock>    # Distinguish "when", "default", and "given" from other statements...
-                    \{  $OWS
                     (?:
-                        when $OWS \(
-                            (?<WHENJUNC> (?: $OWS (?> any | all | none ) $OWS => )?+ )
-                                             $OWS (?<WHENEXPR> (?>(?&PerlExpression)))
-                             $OWS \) $OWS (?>(?<WHENBLOCK> (?&PerlBlock) ))  $OWS
-                        (?{ push @pure_statements, { TYPE => 'when', %+ }; })
+                        when (?<WHENOPEN> $OWS \( $OWS )
+                             (?<WHENEXPR> (?>(?&PerlExpression)))
+                             (?<WHENCLOSE> $OWS \) $OWS )
+                             (?>(?<WHENBLOCK> (?&PerlBlock) ))
+                             (?<WHENPOST> $OWS )
+                             (?{ push @pure_statements, { TYPE => 'when', %+ }; })
                     |
-                        default $OWS (?>(?<DEFBLOCK> (?&PerlBlock) ))  $OWS
-                        (?{ push @pure_statements, { TYPE => 'default', %+ }; })
+                        default (?<DEFPRE>  $OWS )
+                                (?>(?<DEFBLOCK> (?&PerlBlock) ))
+                                (?<DEFPOST> $OWS )
+                            (?{ push @pure_statements, { TYPE => 'default', %+ }; })
                     |
                         (?<NESTEDGIVEN>
                             given \b $OWS  \(
@@ -114,13 +115,17 @@ sub _pure_given_impl { my ($source) = @_;
                                     $OWS \)
                                     $OWS (?>(?<BLOCK>  (?&NestedPureBlock)  )) $OWS
                         )
-                        (?{ push @pure_statements, { TYPE => 'given', %+ }; })
+                            (?{ push @pure_statements, { TYPE => 'given', %+ }; })
                     |
-                        (?! when \b | default \b )
-                        (?>(?<STATEMENT> (?&PerlStatement) ))  $OWS
-                        (?{ push @pure_statements, { TYPE => 'other', %+ }; })
+                        (?! $OWS (?> when | default ) \b )
+                        (?>(?<STATEMENT> (?&PerlStatement)  $OWS ))
+                            (?{ push @pure_statements, { TYPE => 'other', %+ }; })
                     )*+
-                    \}
+
+                    # Possible trailing whitespace at the end of the block...
+                    ( (?>(?<STATEMENT> (?&PerlNWS) ))
+                        (?{ push @pure_statements, { TYPE => 'other', %+ }; })
+                    )?+
                 )
                 (?<NestedPureBlock>    # Non-capturing version of the above
                     \{  $OWS
@@ -163,17 +168,19 @@ sub _pure_given_impl { my ($source) = @_;
         my %matched = %+;
         my $nesting_depth     = 0;
         my $after_a_statement = 0;
+
         return
-              "if (1) { local *_ = \\scalar($matched{EXPR}); if(0){}"
-            . join("\n", map {
+              "if (1) $matched{ws_post_kw} { local *_ = $matched{ws_pre_expr} \\scalar($matched{EXPR}); $matched{ws_pre_close} if(0) $matched{ws_pre_block} }"
+            . join("", map {
                 my $PREFIX = $after_a_statement ? 'if(0){}' : q{};
                 if ($_->{TYPE} eq 'when') {
+                    my $BLOCK = $_->{WHENBLOCK};
                     $after_a_statement = 0;
-                      "$PREFIX elsif (" . _apply_when_magic($_->{WHENEXPR}) . ") $_->{WHENBLOCK}"
+                      "$PREFIX elsif $_->{WHENOPEN}" . _apply_when_magic($_->{WHENEXPR}) . " $_->{WHENCLOSE} $BLOCK $_->{WHENPOST}"
                 }
                 elsif ($_->{TYPE} eq 'default') {
                     $after_a_statement = 0;
-                    "$PREFIX elsif (1) $_->{DEFBLOCK}"
+                    "$PREFIX elsif (1) $_->{DEFPRE} $_->{DEFBLOCK} $_->{DEFPOST}"
                 }
                 elsif ($_->{TYPE} eq 'given') {
                     my $nested = _pure_given_impl($_->{NESTEDGIVEN});
@@ -988,7 +995,7 @@ Switch::Back - C<given>/C<when> for a post-C<given>/C<when> Perl
 
 =head1 VERSION
 
-This document describes Switch::Back version 0.000002
+This document describes Switch::Back version 0.000003
 
 
 =head1 SYNOPSIS
